; ---------------------------------------------------------------------------
; VidDots - an experiment in pixel addressing and data structures
;
; This program displays a set of traveling lines that bounce when they hit
; the boundries of the display window.  The endpoints of the lines are
; stored in a simple structure that hold position, direction vector, and
; 'draw color' (on/off) information.  As the head of the line advances leaving
; a trail of set pixels, the end of the line also advances in the same path,
; a few pixels behind, erasing them.  By keeping the end-point information in
; a structure it is a simple matter to change and/or increase the number of
; displayed lines.
; 
; Notes:
;
;   Ram Usage
;     Page 0 - code
;     Page 1 - main
;     Page 2 - video RAM
;     Page 3 - video RAM
; 
;   end-point (dot) structure
;     color (0x01 = on, 0x00 = off)
;     X position (0x00 - 0x3F)
;     Y position (0x00-0x3F)
;     deltaX (2's complement)
;     deltaY (2's complement)
; 
; The "DrwPt" routine was copied from "viddots-asm.txt" and enhanced to use
; 'D' bit-0 to indicate if the specified pixel is to be set or cleared, to 
; ensure that RB was restored correctly before returning, and to use the
; SCRT call/return mechanism rather than a dedicated register.
; 
; In order to use 'D' as described above, the SCRT routines also needed 
; revision to preserve D & DF.
; 
; All of the extra code expanded the last used address beyond 0x0100, into
; the display area. So, the main routine was relocated out to ram page 3.
; 
; ng 1/31/2005
; ---------------------------------------------------------------------------

PgRam	EQU	#02
PgData	EQU	#00
PgMain	EQU	#01

; ------------------- init registers, jmp to main
		GHI	R0		; a tricky 1-byte way to set D=0
		PHI	R1
		PHI	R2
		PHI	R4
		PHI	R5

		LDI	Stack
		PLO	R2

		LDI	DmaPrep
		PLO	R1

		LDI	SCall	; R4 aka CALL
		PLO	R4

		LDI	SRet	; R5 aka RETURN
		PLO	R5
		
		LDI	PgMain
		PHI	R3
		LDI	Main
		PLO	R3
		
		SEP	R3		; launch Main
		
; ..........................
; SCRT "Call" routine
; modified from the RCA example to preserve D,DF
; for parameter passing
;
exCall	SEP R3	; jump to M(R3), reset SCall addr
SCall	STXD	; Save D
		RSHR	; Save DF
		STXD	; .
		GLO R3	; R6 <= R3
		PLO R6	; .
		GHI R3	; .
		PHI R6	; .
		LDA R6	; R3 <= M(R6)
		PHI R3	; .
		LDA R6	; .
		PLO R3	; .
		INC	R2	; Restore DF
		LDXA	; .
		RSHL	; .
		LDX		; Restore D
		BR  exCall

; ..........................
; SCRT "Return" routine
; modified from the RCA example to preserve D,DF
; for parameter passing
exRet	SEP R3	; jump to M(R3), reset SRet addr
SRet	STXD	; Save D
		RSHR	; Save DF
		STXD	; .
		GLO R6	; R3 <= R6
		PLO R3	; .
		GHI R6	; .
		PHI R3	; . 
		INC R2	; Restore DF
		LDXA	; .
		RSHL	; .
		LDX		; Restore D
		BR  exRet

; ------------------- intr, dma handler
DmaRtn	LDXA
		RET

DmaPrep	DEC	R2
		SAV
		DEC	R2
		STR	R2
		NOP
		NOP
		NOP
		LDI PgRam	; RAM Page to display
		PHI R0
		LDI #00
		PLO R0
;		... each display-line is repeated
;		... creating a 64x64 pixel display
DmaRow	GLO R0		; Next line
		SEX R2
		SEX R2
		DEC R0		; Repeat line data
		PLO R0		; ..
		SEX R2
		GLO R0		; Next line
		PLO R0
		SEX R2
		DEC R0		; Repeat line data
		PLO R0		; ..
		BN1 DmaRow

		BR  DmaRtn
		
; -------------------------------------------------
; Draw-Point
; 
; RB is assumed to point to a 2-byte structure
; specifying the X and Y coordinate of the point
; to be set.
; The value of 'D' bit 0 determines if the video
; bit is set or cleared.
;
; Assuming a 64x64 display the byte to modify is
; calculated as [row: 0x0100 + 8*y] + [column: x/8]
; and the bit position to modify is (x mod 8)
;
; Also used are:
;   RD, vid ram pointer
;   RE, scratch pad
;
; Also assumed is the video ram address of 0x0100
;
DrwPt	STXD		; save 'D' for later
		INC	RB		; get y value
		LDN RB		; ..
		DEC	RB		; reset RB
		PLO RD
		
		LDI	#00		; . RD << 3  (y*8)
		PHI	RD		; . (a 16-bit shift)
		LDI	#03		; .
		PLO	RE		; .
				
Shft8	GLO RD		; .
		SHL			; .
		PLO	RD		; .
		GHI	RD		; .
		SHLC		; .
		PHI	RD		; .
		DEC	RE		; .
		GLO	RE		; .
		BNZ	Shft8	; .
		
		LDN RB		; get x value
		SHR			; div by 8
		SHR
		SHR

		STR	R2		; add with 16-bit value in RD
		GLO	RD
		ADD
		PLO	RD		; put result in RD.0
		GHI RD
		ADCI #00
		PHI	RD
		
		GHI	RD		; add video ram page offset to RD
		ADI	PgRam
		PHI	RD
		
		LDN RB		; preform (x MOD 8)
		ANI #07
		PLO RE		; save in RE.0
		
		LDI #80		; generate bit mask
		PHI	RE
MkMask	GLO RE
		BZ	MaskOk
		GHI RE
		SHR
		PHI RE
		DEC RE
		BR	MkMask
		
MaskOk	INC	R2		; Recall 'D'
		LDX
		ANI	#01
		BZ	DrwOff		
DrwOn	GHI	RE		; OR mask with current vid RAM byte
		STR R2
		LDN RD
		OR
		STR RD
		SEP	R5		; SCRT SRet (return)
DrwOff	GHI	RE
		XRI	#FF
		STR R2
		LDN RD
		AND
		STR RD
		SEP R5		; SCRT SRet (return)

; -------------------------------------------------
; Erases the Memory page specified in RF.1
;
MemPgErase	LDI	#00
			PLO	RF
ers01		LDI	#00
			STR	RF
			INC	RF
			GLO RF
			BNZ	ers01
			SEP	R5

; -------------------------------------
; Data storage
;
nDots	EQU	14

PtPair	DB	#01,#0A,#08,#01,#01
		DB	#00,#04,#02,#01,#01
		
		DB	#01,#1F,#36,#01,#FF
		DB	#00,#19,#3C,#01,#FF
		
		DB	#01,#27,#1C,#01,#01
		DB	#00,#21,#16,#01,#01
		
		DB	#01,#15,#26,#01,#FF
		DB	#00,#01,#3A,#01,#FF

		DB	#01,#2A,#26,#FF,#01
		DB	#00,#3E,#12,#FF,#01
		
		DB	#01,#30,#08,#FF,#FF
		DB	#00,#34,#0C,#FF,#FF
		
		DB	#01,#10,#3A,#FF,#FF
		DB	#00,#14,#3E,#FF,#FF

; -------------------------------------
; Reserve stack space
;
		DB #AA,#AA,#AA,#AA,#AA,#AA,#AA
Stack	DB #AA

; -------------------------------------
; Ensure "Main" is on page 1

		DB 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
		DB 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; -------------------------------------
; Program Main
;
Main	SEX R2
		IN1		; start VDG

;		........ clear video ram pages
		LDI	PgRam
		PHI	RF
		SEP R4
		DW	MemPgErase
		SEP R4
		DW	MemPgErase
		
; draw dots
; update dot position
;   if hit boundries, revise values

ForEvr	LDI	PgData
		PHI RB	; RB, X-Y data pair storage
		LDI	PtPair
		PLO	RB
		
		LDI	nDots
		PLO	RC
		
NxtOne	LDA	RB		; get 'color' value, and point to xy pos 
;					; within the structure
		SEP	R4		; then draw it
		DW	DrwPt
		
		INC RB		; position to next structure
		INC RB
		INC RB
		INC RB
		DEC	RC
		GLO	RC
		BNZ	NxtOne
		
		LDI	PgData
		PHI RB	; RB, X-Y data pair storage
		LDI	PtPair
		PLO	RB
		
		LDI	nDots
		PLO	RC
NxtTwo	INC RB		; skip 'color'
		LDA	RB
		PLO	RD		; RD.0 = x
		LDA RB
		PHI RD		; RD.1 = y
		
		LDA RB		; RD.0 += delta-x
		STR	R2		; 
		GLO	RD
		ADD
		PLO	RD
		
		LDN	RB		; RD.1 += delta-y
		STR	R2
		GHI	RD
		ADD
		PHI	RD
		
		DEC	RB		; move to dx-value in struct
		GLO	RD
		SEP	R4
		DW	HitTest
		
		INC	RB		; move to dy-value in struct
		GHI	RD
		SEP	R4
		DW	HitTest
		
		INC RB		; position to next structure
		DEC	RC
		GLO	RC
		BNZ	NxtTwo
		
		BR	ForEvr		
		
; -------------------------------------------------
; Test if "dot' value in 'D' hit the display boundry
; assumes RB points to the appropriate dx or dy
;
HitTest STXD		; push 'D' onto stack to preserve it
		ANI	#C0		; is 0 < new value < 3f ?
		BZ	ok100	; no, then save 'D' as updated value
		LDN	RB		; yes, figure out what to do
		ANI	#80		
		BZ	fix01	;  ... was positive delta
		LDI	#01		;  ... was negative, make positive
		STR	RB
		LDI	#00		;  ... reset to top or left boundry
		BR	ok101
fix01	LDI	#FF		;  ... was negative, make positive
		STR	RB
		LDI	#3F		;  ... reset to right or bottom boundry
ok101	INC	R2		;  ... replace preserved 'D' value
		STXD		
ok100	INC	R2		; pop 'D' value off stack
		LDX
		DEC	RB		; move to value position within struct
		DEC	RB
		STR	RB		; save value
		INC RB		; restore RB
		INC RB
		SEP R5		; SCRT Return

